# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Optional, TYPE_CHECKING, cast
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.settings import settings
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy
from azure.mgmt.core.tools import get_arm_endpoints

from ._configuration import IoTFirmwareDefenseMgmtClientConfiguration
from ._utils.serialization import Deserializer, Serializer
from .operations import (
    BinaryHardeningOperations,
    CryptoCertificatesOperations,
    CryptoKeysOperations,
    CvesOperations,
    FirmwaresOperations,
    Operations,
    PasswordHashesOperations,
    SbomComponentsOperations,
    SummariesOperations,
    UsageMetricsOperations,
    WorkspacesOperations,
)

if TYPE_CHECKING:
    from azure.core.credentials import TokenCredential


class IoTFirmwareDefenseMgmtClient:  # pylint: disable=too-many-instance-attributes
    """Firmware & IoT Security REST API.

    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.iotfirmwaredefense.operations.Operations
    :ivar firmwares: FirmwaresOperations operations
    :vartype firmwares: azure.mgmt.iotfirmwaredefense.operations.FirmwaresOperations
    :ivar workspaces: WorkspacesOperations operations
    :vartype workspaces: azure.mgmt.iotfirmwaredefense.operations.WorkspacesOperations
    :ivar binary_hardening: BinaryHardeningOperations operations
    :vartype binary_hardening: azure.mgmt.iotfirmwaredefense.operations.BinaryHardeningOperations
    :ivar crypto_certificates: CryptoCertificatesOperations operations
    :vartype crypto_certificates:
     azure.mgmt.iotfirmwaredefense.operations.CryptoCertificatesOperations
    :ivar crypto_keys: CryptoKeysOperations operations
    :vartype crypto_keys: azure.mgmt.iotfirmwaredefense.operations.CryptoKeysOperations
    :ivar cves: CvesOperations operations
    :vartype cves: azure.mgmt.iotfirmwaredefense.operations.CvesOperations
    :ivar password_hashes: PasswordHashesOperations operations
    :vartype password_hashes: azure.mgmt.iotfirmwaredefense.operations.PasswordHashesOperations
    :ivar sbom_components: SbomComponentsOperations operations
    :vartype sbom_components: azure.mgmt.iotfirmwaredefense.operations.SbomComponentsOperations
    :ivar summaries: SummariesOperations operations
    :vartype summaries: azure.mgmt.iotfirmwaredefense.operations.SummariesOperations
    :ivar usage_metrics: UsageMetricsOperations operations
    :vartype usage_metrics: azure.mgmt.iotfirmwaredefense.operations.UsageMetricsOperations
    :param credential: Credential used to authenticate requests to the service. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
    :type subscription_id: str
    :param base_url: Service host. Default value is None.
    :type base_url: str
    :keyword api_version: The API version to use for this operation. Default value is
     "2025-04-01-preview". Note that overriding this default value may result in unsupported
     behavior.
    :paramtype api_version: str
    """

    def __init__(
        self, credential: "TokenCredential", subscription_id: str, base_url: Optional[str] = None, **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        _cloud = kwargs.pop("cloud_setting", None) or settings.current.azure_cloud  # type: ignore
        _endpoints = get_arm_endpoints(_cloud)
        if not base_url:
            base_url = _endpoints["resource_manager"]
        credential_scopes = kwargs.pop("credential_scopes", _endpoints["credential_scopes"])
        self._config = IoTFirmwareDefenseMgmtClientConfiguration(
            credential=credential,
            subscription_id=subscription_id,
            base_url=cast(str, base_url),
            credential_scopes=credential_scopes,
            **kwargs
        )

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=cast(str, _endpoint), policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.firmwares = FirmwaresOperations(self._client, self._config, self._serialize, self._deserialize)
        self.workspaces = WorkspacesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.binary_hardening = BinaryHardeningOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.crypto_certificates = CryptoCertificatesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.crypto_keys = CryptoKeysOperations(self._client, self._config, self._serialize, self._deserialize)
        self.cves = CvesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.password_hashes = PasswordHashesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.sbom_components = SbomComponentsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.summaries = SummariesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.usage_metrics = UsageMetricsOperations(self._client, self._config, self._serialize, self._deserialize)

    def send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client.send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)
